@@ -25,9 +25,9 @@ import javax.servlet.ReadListener;
 import org.apache.tomcat.util.buf.ByteChunk;
 import org.apache.tomcat.util.buf.MessageBytes;
 import org.apache.tomcat.util.buf.UDecoder;
-import org.apache.tomcat.util.http.Cookies;
 import org.apache.tomcat.util.http.MimeHeaders;
 import org.apache.tomcat.util.http.Parameters;
+import org.apache.tomcat.util.http.ServerCookies;
 import org.apache.tomcat.util.res.StringManager;
 
 /**
@@ -62,6 +62,8 @@ public final class Request {
     private static final StringManager sm =
             StringManager.getManager(Constants.Package);
 
+    // Expected maximum typica number of cookies per request.
+    private static final int INITIAL_COOKIE_SIZE = 4;
 
     // ----------------------------------------------------------- Constructors
 
@@ -121,17 +123,19 @@ public final class Request {
     private long contentLength = -1;
     private MessageBytes contentTypeMB = null;
     private String charEncoding = null;
-    private final Cookies cookies = new Cookies(headers);
+
+    private final ServerCookies serverCookies = new ServerCookies(INITIAL_COOKIE_SIZE);
     private final Parameters parameters = new Parameters();
 
-    private final MessageBytes remoteUser=MessageBytes.newInstance();
-    private final MessageBytes authType=MessageBytes.newInstance();
-    private final HashMap<String,Object> attributes=new HashMap<>();
+    private final MessageBytes remoteUser = MessageBytes.newInstance();
+    private boolean remoteUserNeedsAuthorization = false;
+    private final MessageBytes authType = MessageBytes.newInstance();
+    private final HashMap<String,Object> attributes = new HashMap<>();
 
     private Response response;
     private ActionHook hook;
 
-    private int bytesRead=0;
+    private long bytesRead=0;
     // Time of the request - useful to avoid repeated calls to System.currentTime
     private long startTime = -1;
     private int available = 0;
@@ -381,15 +385,13 @@ public final class Request {
 
     // -------------------- Cookies --------------------
 
-
-    public Cookies getCookies() {
-        return cookies;
+    public ServerCookies getCookies() {
+        return serverCookies;
     }
 
 
     // -------------------- Parameters --------------------
 
-
     public Parameters getParameters() {
         return parameters;
     }
@@ -414,6 +416,14 @@ public final class Request {
         return remoteUser;
     }
 
+    public boolean getRemoteUserNeedsAuthorization() {
+        return remoteUserNeedsAuthorization;
+    }
+
+    public void setRemoteUserNeedsAuthorization(boolean remoteUserNeedsAuthorization) {
+        this.remoteUserNeedsAuthorization = remoteUserNeedsAuthorization;
+    }
+
     public MessageBytes getAuthType() {
         return authType;
     }
@@ -450,7 +460,7 @@ public final class Request {
      *
      * The buffer is owned by the protocol implementation - it will be reused on the next read.
      * The Adapter must either process the data in place or copy it to a separate buffer if it needs
-     * to hold it. In most cases this is done during byte->char conversions or via InputStream. Unlike
+     * to hold it. In most cases this is done during byte-&gt;char conversions or via InputStream. Unlike
      * InputStream, this interface allows the app to process data in place, without copy.
      *
      */
@@ -528,7 +538,7 @@ public final class Request {
         remotePort = -1;
         available = 0;
 
-        cookies.recycle();
+        serverCookies.recycle();
         parameters.recycle();
 
         uriMB.recycle();
@@ -541,6 +551,7 @@ public final class Request {
 
         instanceId.recycle();
         remoteUser.recycle();
+        remoteUserNeedsAuthorization = false;
         authType.recycle();
         attributes.clear();
 
@@ -559,7 +570,7 @@ public final class Request {
         return reqProcessorMX;
     }
 
-    public int getBytesRead() {
+    public long getBytesRead() {
         return bytesRead;
     }
 
