@@ -129,11 +129,6 @@ public class NioEndpoint extends AbstractEndpoint<NioChannel> {
      */
     private SynchronizedStack<SocketProcessor> processorCache;
 
-    /**
-     * Cache for key attachment objects
-     */
-    private SynchronizedStack<KeyAttachment> keyCache;
-
     /**
      * Cache for poller events
      */
@@ -304,7 +299,6 @@ public class NioEndpoint extends AbstractEndpoint<NioChannel> {
     }
 
     protected void releaseCaches() {
-        this.keyCache.clear();
         this.nioChannels.clear();
         this.processorCache.clear();
         if ( handler != null ) handler.recycle();
@@ -403,8 +397,6 @@ public class NioEndpoint extends AbstractEndpoint<NioChannel> {
 
             processorCache = new SynchronizedStack<>(SynchronizedStack.DEFAULT_SIZE,
                     socketProperties.getProcessorCache());
-            keyCache = new SynchronizedStack<>(SynchronizedStack.DEFAULT_SIZE,
-                            socketProperties.getKeyCache());
             eventCache = new SynchronizedStack<>(SynchronizedStack.DEFAULT_SIZE,
                             socketProperties.getEventCache());
             nioChannels = new SynchronizedStack<>(SynchronizedStack.DEFAULT_SIZE,
@@ -455,7 +447,6 @@ public class NioEndpoint extends AbstractEndpoint<NioChannel> {
             }
             shutdownExecutor();
             eventCache.clear();
-            keyCache.clear();
             nioChannels.clear();
             processorCache.clear();
         }
@@ -588,7 +579,9 @@ public class NioEndpoint extends AbstractEndpoint<NioChannel> {
         engine.setEnabledCipherSuites(enabledCiphers);
         engine.setEnabledProtocols(enabledProtocols);
 
+        configureUseServerCipherSuitesOrder(engine);
         handler.onCreateSSLEngine(engine);
+
         return engine;
     }
 
@@ -900,7 +893,7 @@ public class NioEndpoint extends AbstractEndpoint<NioChannel> {
             }
             addEvent(r);
             if (close) {
-                NioEndpoint.KeyAttachment ka = (NioEndpoint.KeyAttachment)socket.getAttachment(false);
+                NioEndpoint.KeyAttachment ka = (NioEndpoint.KeyAttachment)socket.getAttachment();
                 processSocket(ka, SocketStatus.STOP, false);
             }
         }
@@ -938,9 +931,9 @@ public class NioEndpoint extends AbstractEndpoint<NioChannel> {
          */
         public void register(final NioChannel socket) {
             socket.setPoller(this);
-            KeyAttachment key = keyCache.pop();
-            final KeyAttachment ka = key!=null?key:new KeyAttachment(socket);
-            ka.reset(this,socket,getSocketProperties().getSoTimeout());
+            KeyAttachment ka = new KeyAttachment(socket);
+            ka.setPoller(this);
+            ka.setTimeout(getSocketProperties().getSoTimeout());
             ka.setKeepAliveLeft(NioEndpoint.this.getMaxKeepAliveRequests());
             ka.setSecure(isSSLEnabled());
             PollerEvent r = eventCache.pop();
@@ -950,22 +943,23 @@ public class NioEndpoint extends AbstractEndpoint<NioChannel> {
             addEvent(r);
         }
 
-        public void cancelledKey(SelectionKey key, SocketStatus status) {
+        public KeyAttachment cancelledKey(SelectionKey key, SocketStatus status) {
+            KeyAttachment ka = null;
             try {
-                if ( key == null ) return;//nothing to do
-                KeyAttachment ka = (KeyAttachment) key.attachment();
+                if ( key == null ) return null;//nothing to do
+                ka = (KeyAttachment) key.attachment();
                 if (ka != null && ka.isComet() && status != null) {
                     ka.setComet(false);//to avoid a loop
                     if (status == SocketStatus.TIMEOUT ) {
                         if (processSocket(ka, status, true)) {
-                            return; // don't close on comet timeout
+                            return null; // don't close on comet timeout
                         }
                     } else {
-                        // Don't dispatch if the lines below are canceling the key
+                        // Don't dispatch if the lines below are cancelling the key
                         processSocket(ka, status, false);
                     }
                 }
-                key.attach(null);
+                ka = (KeyAttachment) key.attach(null);
                 if (ka!=null) handler.release(ka);
                 else handler.release((SocketChannel)key.channel());
                 if (key.isValid()) key.cancel();
@@ -997,14 +991,14 @@ public class NioEndpoint extends AbstractEndpoint<NioChannel> {
                     }
                 } catch (Exception ignore) {
                 }
-                if (ka!=null) {
-                    ka.reset();
+                if (ka != null) {
                     countDownConnection();
                 }
             } catch (Throwable e) {
                 ExceptionUtils.handleThrowable(e);
                 if (log.isDebugEnabled()) log.error("",e);
             }
+            return ka;
         }
         /**
          * The background thread that listens for incoming TCP/IP connections and
@@ -1062,16 +1056,6 @@ public class NioEndpoint extends AbstractEndpoint<NioChannel> {
                             }
                             break;
                         }
-                    } catch ( NullPointerException x ) {
-                        //sun bug 5076772 on windows JDK 1.5
-                        if ( log.isDebugEnabled() ) log.debug("Possibly encountered sun bug 5076772 on windows JDK 1.5",x);
-                        if ( wakeupCounter == null || selector == null ) throw x;
-                        continue;
-                    } catch ( CancelledKeyException x ) {
-                        //sun bug 5076772 on windows JDK 1.5
-                        if ( log.isDebugEnabled() ) log.debug("Possibly encountered sun bug 5076772 on windows JDK 1.5",x);
-                        if ( wakeupCounter == null || selector == null ) throw x;
-                        continue;
                     } catch (Throwable x) {
                         ExceptionUtils.handleThrowable(x);
                         log.error("",x);
@@ -1359,39 +1343,6 @@ public class NioEndpoint extends AbstractEndpoint<NioChannel> {
             super(channel);
         }
 
-        public void reset(Poller poller, NioChannel channel, long soTimeout) {
-            super.reset(channel, soTimeout);
-
-            cometNotify = false;
-            interestOps = 0;
-            this.poller = poller;
-            sendfileData = null;
-            if (readLatch != null) {
-                try {
-                    for (int i = 0; i < (int) readLatch.getCount(); i++) {
-                        readLatch.countDown();
-                    }
-                } catch (Exception ignore) {
-                }
-            }
-            readLatch = null;
-            sendfileData = null;
-            if (writeLatch != null) {
-                try {
-                    for (int i = 0; i < (int) writeLatch.getCount(); i++) {
-                        writeLatch.countDown();
-                    }
-                } catch (Exception ignore) {
-                }
-            }
-            writeLatch = null;
-            setWriteTimeout(soTimeout);
-        }
-
-        public void reset() {
-            reset(null,null,-1);
-        }
-
         public Poller getPoller() { return poller;}
         public void setPoller(Poller poller){this.poller = poller;}
         public void setCometNotify(boolean notify) { this.cometNotify = notify; }
@@ -1439,7 +1390,7 @@ public class NioEndpoint extends AbstractEndpoint<NioChannel> {
         private boolean cometNotify = false;
         private CountDownLatch readLatch = null;
         private CountDownLatch writeLatch = null;
-        private SendfileData sendfileData = null;
+        private volatile SendfileData sendfileData = null;
         private long writeTimeout = -1;
 
     }
@@ -1569,21 +1520,22 @@ public class NioEndpoint extends AbstractEndpoint<NioChannel> {
                         // Close socket and pool
                         try {
                             ka.setComet(false);
-                            socket.getPoller().cancelledKey(key, SocketStatus.ERROR);
-                            if (running && !paused) {
-                                nioChannels.push(socket);
-                            }
-                            socket = null;
-                            if (running && !paused) {
-                                keyCache.push(ka);
+                            if (socket.getPoller().cancelledKey(key, SocketStatus.ERROR) != null) {
+                                // SocketWrapper (attachment) was removed from the
+                                // key - recycle both. This can only happen once
+                                // per attempted closure so it is used to determine
+                                // whether or not to return socket and ka to
+                                // their respective caches. We do NOT want to do
+                                // this more than once - see BZ 57340.
+                                if (running && !paused) {
+                                    nioChannels.push(socket);
+                                }
+                                socket = null;
                             }
                             ka = null;
                         } catch (Exception x) {
                             log.error("",x);
                         }
-                    } else if (state == SocketState.LONG && ka.isAsync() && ka.interestOps() > 0) {
-                        //we are async, and we are interested in operations
-                        ka.getPoller().add(socket, ka.interestOps());
                     }
                 } else if (handshake == -1 ) {
                     if (key != null) {
@@ -1593,9 +1545,6 @@ public class NioEndpoint extends AbstractEndpoint<NioChannel> {
                         nioChannels.push(socket);
                     }
                     socket = null;
-                    if (running && !paused) {
-                        keyCache.push(ka);
-                    }
                     ka = null;
                 } else {
                     ka.getPoller().add(socket,handshake);
@@ -1644,11 +1593,11 @@ public class NioEndpoint extends AbstractEndpoint<NioChannel> {
      */
     public static class SendfileData {
         // File
-        public String fileName;
-        public FileChannel fchannel;
-        public long pos;
-        public long length;
+        public volatile String fileName;
+        public volatile FileChannel fchannel;
+        public volatile long pos;
+        public volatile long length;
         // KeepAlive flag
-        public boolean keepAlive;
+        public volatile boolean keepAlive;
     }
 }
