@@ -17,6 +17,7 @@
 package org.apache.catalina.loader;
 
 import java.io.InputStream;
+import java.util.concurrent.Executor;
 import java.util.concurrent.atomic.AtomicInteger;
 import java.util.logging.Filter;
 import java.util.logging.LogManager;
@@ -24,7 +25,6 @@ import java.util.logging.LogRecord;
 
 import javax.servlet.http.HttpServletResponse;
 
-
 import org.junit.Assert;
 import org.junit.Test;
 
@@ -34,6 +34,7 @@ import org.apache.catalina.core.StandardHost;
 import org.apache.catalina.startup.Tomcat;
 import org.apache.catalina.startup.TomcatBaseTest;
 import org.apache.tomcat.util.buf.ByteChunk;
+import org.apache.tomcat.util.threads.ThreadPoolExecutor;
 
 public class TestWebappClassLoaderThreadLocalMemoryLeak extends TomcatBaseTest {
 
@@ -45,20 +46,21 @@ public class TestWebappClassLoaderThreadLocalMemoryLeak extends TomcatBaseTest {
         tomcat.getServer().addLifecycleListener(
                 new JreMemoryLeakPreventionListener());
 
-        // Must have a real docBase - just use temp
-        Context ctx = tomcat.addContext("",
-                System.getProperty("java.io.tmpdir"));
+        // No file system docBase required
+        Context ctx = tomcat.addContext("", null);
 
         Tomcat.addServlet(ctx, "leakServlet1",
                 "org.apache.tomcat.unittest.TesterLeakingServlet1");
         ctx.addServletMapping("/leak1", "leakServlet1");
 
-
         tomcat.start();
 
+        Executor executor = tomcat.getConnector().getProtocolHandler().getExecutor();
+        ((ThreadPoolExecutor) executor).setThreadRenewalDelay(-1);
+
         // Configure logging filter to check leak message appears
         LogValidationFilter f = new LogValidationFilter(
-                "The web application [] created a ThreadLocal with key of");
+                "The web application [ROOT] created a ThreadLocal with key of");
         LogManager.getLogManager().getLogger(
                 "org.apache.catalina.loader.WebappClassLoaderBase").setFilter(f);
 
@@ -100,9 +102,8 @@ public class TestWebappClassLoaderThreadLocalMemoryLeak extends TomcatBaseTest {
         tomcat.getServer().addLifecycleListener(
                 new JreMemoryLeakPreventionListener());
 
-        // Must have a real docBase - just use temp
-        Context ctx = tomcat.addContext("",
-                System.getProperty("java.io.tmpdir"));
+        // No file system docBase required
+        Context ctx = tomcat.addContext("", null);
 
         Tomcat.addServlet(ctx, "leakServlet2",
                 "org.apache.tomcat.unittest.TesterLeakingServlet2");
@@ -110,9 +111,12 @@ public class TestWebappClassLoaderThreadLocalMemoryLeak extends TomcatBaseTest {
 
         tomcat.start();
 
+        Executor executor = tomcat.getConnector().getProtocolHandler().getExecutor();
+        ((ThreadPoolExecutor) executor).setThreadRenewalDelay(-1);
+
         // Configure logging filter to check leak message appears
         LogValidationFilter f = new LogValidationFilter(
-                "The web application [] created a ThreadLocal with key of");
+                "The web application [ROOT] created a ThreadLocal with key of");
         LogManager.getLogManager().getLogger(
                 "org.apache.catalina.loader.WebappClassLoaderBase").setFilter(f);
 
